import logging
import paths as pt
from pathlib import Path
import os

def setup_logger(name):
    # Create a custom logger
    logger = logging.getLogger(name)

    # Setup log dir
    logdir = Path.joinpath(pt.ROOT_DIR, 'logs')
    if not os.path.exists(logdir):
        os.makedirs(logdir)
    logfile = Path.joinpath(logdir, name + ".log")
    print(f"Logs for {name} will be saved to: {logfile}")

    # Create handlers and set their level
    s_handler = logging.StreamHandler()
    f_handler = logging.FileHandler(logfile)
    s_handler.setLevel(logging.WARNING)
    f_handler.setLevel(logging.DEBUG)

    # Create formatters and add it to handlers
    c_format = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
    f_format = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    s_handler.setFormatter(c_format)
    f_handler.setFormatter(f_format)

    # Add handlers to the logger
    logger.addHandler(s_handler)
    logger.addHandler(f_handler)

    # Set global level
    logger.setLevel(logging.DEBUG)

    return logger