import pandas as pd
import numpy as np
from sklearn.datasets import make_classification
from sklearn.preprocessing import LabelEncoder
from tools import preprocessor

def test_series_to_list():
    data = np.array([1, 2, 3]) 
    ser = pd.Series(data)
    res = preprocessor.series_to_list(ser)
    assert len(res) == 3

def test_sample():
    X, y = make_classification(n_features=2, n_redundant=0, n_informative=2,
                               random_state=0, n_clusters_per_class=1)
    X_sampled, y_sampled = preprocessor.sample(X, y, 1)
    assert X_sampled.shape == (1, 2)
    assert y_sampled.shape == (1,)

def test_transpose_to_list():
    X = np.random.rand(100, 2)
    res = preprocessor.transpose_to_list(X)
    assert len(res) == 2
    assert len(res[0]) == 100
    assert len(res[1]) == 100

def test_get_ats_list():
    df = pd.DataFrame()
    df['Age'] = np.random.randint(0, 5, 10)
    df['DevISOClass'] = ['120606'] * 10
    df['CitizenId'] = np.random.randint(0, 10, 10)
    res = preprocessor.get_ats_list(df)
    assert set(['Ats']).issubset(res.columns)

def test_get_class_weight():
    neg = 100
    pos = 50
    res = preprocessor.get_class_weight(neg, pos)
    assert res == {0: 0.75, 1: 1.5}

def test_prepare_data_for_embedder():
    df = pd.DataFrame()
    df['Age'] = np.random.randint(0, 5, 10)
    df['Complete'] = np.random.randint(0, 1, 10)
    xt, xv, yt, yv, lbl = preprocessor.prepare_data_for_embedder(df, 'Complete', 0.9)
    assert len(xt) == 9 
    assert len(xv) == 1
    assert len(yt) == 9
    assert len(yv) == 1
    assert isinstance(lbl[0], LabelEncoder)
    
def test_process_target():
    y = ['Complete'] * 100
    res = preprocessor.process_target(y)
    assert len(res) == 100
    assert np.count_nonzero(res) == 0

def test_split_cat_columns():
    df = pd.DataFrame()
    df['Ats'] = ['120606,120608,120610'] * 10
    res = preprocessor.split_cat_columns(df, 'Ats', 'Ats', 3)
    assert res.shape == (10, 3)
    assert list(res.columns) == ['1Ats', '2Ats', '3Ats']

def test_extract_cat_count():
    df = pd.DataFrame()
    df['1Ats'] = ['120606'] * 10
    cols_ats = ['1Ats']
    unique_ats = ['120606']
    res = preprocessor.extract_cat_count(df, unique_ats, cols_ats, 'Ats_')
    assert res.shape == (10, 1)
    assert list(res.columns) == ['Ats_120606']

def test_replace_cat_values():
    df = pd.DataFrame(np.array([['120606']]), columns=['1Ats'])
    ats = pd.DataFrame(np.array([['120606', 'Rollatorer']]),
                       columns=['ats_id', 'ats_name'])
    res = preprocessor.replace_cat_values(df, ats)
    assert res['1Ats'].iloc[0] == 'Rollatorer'